@@ -51,6 +51,7 @@ int cl_solid = 0;		// 1 = dsolid, 2 = asolid, 3 = solid
 namespace {
 
 LZ_Encoder * encoder = 0;	// local vars needed by add_member
+const char * archive_namep = 0;
 int outfd = -1;
 int gretval = 0;
 
@@ -97,7 +98,7 @@ bool copy_file( const int infd, const int outfd, const long long max_size = -1 )
 
 
 /* Check archive type. If success, leave fd file pos at 0.
-   If remove_eof, remove EOF blocks, and leave fd file pos at EOF. */
+   If remove_eof, leave fd file pos at beginning of the EOF blocks. */
 bool check_appendable( const int fd, const bool remove_eof )
   {
   struct stat st;
@@ -144,15 +145,33 @@ bool check_appendable( const int fd, const bool remove_eof )
   crc ^= 0xFFFFFFFFU;
   if( crc != data_crc ) return false;
 
-  if( remove_eof )
-    { if( lseek( fd, end - member_size, SEEK_SET ) != end - member_size ||
-        ftruncate( fd, end - member_size ) != 0 ) return false; }
-  else
-    if( lseek( fd, 0, SEEK_SET ) != 0 ) return false;
-  return true;
+  const long long pos = remove_eof ? end - member_size : 0;
+  return ( lseek( fd, pos, SEEK_SET ) == pos );
   }
 
 
+class File_is_archive
+  {
+  dev_t archive_dev;
+  ino_t archive_ino;
+  bool initialized;
+public:
+  File_is_archive() : initialized( false ) {}
+  bool init()
+    {
+    struct stat st;
+    if( fstat( outfd, &st ) != 0 ) return false;
+    if( S_ISREG( st.st_mode ) )
+      { archive_dev = st.st_dev; archive_ino = st.st_ino; initialized = true; }
+    return true;
+    }
+  bool operator()( const struct stat & st ) const
+    {
+    return initialized && archive_dev == st.st_dev && archive_ino == st.st_ino;
+    }
+  } file_is_archive;
+
+
 bool archive_write( const uint8_t * const buf, const int size )
   {
   if( !encoder )					// uncompressed
@@ -166,9 +185,10 @@ bool archive_write( const uint8_t * const buf, const int size )
     const int wr = LZ_compress_write( encoder, buf + sz, size - sz );
     if( wr < 0 ) internal_error( "library error (LZ_compress_write)." );
     sz += wr;
+    if( sz >= size && size > 0 ) break;		// minimize dictionary size
     const int rd = LZ_compress_read( encoder, obuf, obuf_size );
     if( rd < 0 ) internal_error( "library error (LZ_compress_read)." );
-    if( rd == 0 && sz == size ) break;
+    if( rd == 0 && sz >= size ) break;
     if( writeblock( outfd, obuf, rd ) != rd ) return false;
     }
   if( LZ_compress_finished( encoder ) == 1 &&
@@ -317,6 +337,9 @@ int add_member( const char * const filename, const struct stat *,
   if( lstat( filename, &st ) != 0 )
     { show_file_error( filename, "Can't stat input file", errno );
       gretval = 1; return 0; }
+  if( file_is_archive( st ) )
+    { show_file_error( archive_namep, "File is the archive; not dumped." );
+      return 0; }
   Extended extended;		// metadata for extended records
   Tar_header header;
   std::memset( header, 0, header_size );
@@ -458,6 +481,8 @@ int concatenate( const std::string & archive_name, const Arg_parser & parser,
     { show_error( "'--concatenate' is incompatible with '-f -'.", 0, true );
       return 1; }
   if( ( outfd = open_outstream( archive_name, false ) ) < 0 ) return 1;
+  if( !file_is_archive.init() )
+    { show_file_error( archive_name.c_str(), "Can't stat", errno ); return 1; }
 
   int retval = 0;
   for( int i = 0; i < parser.arguments(); ++i )		// copy archives
@@ -471,6 +496,10 @@ int concatenate( const std::string & archive_name, const Arg_parser & parser,
     if( !check_appendable( infd, false ) )
       { show_file_error( filename, "Not an appendable tar.lz archive." );
         close( infd ); retval = 2; break; }
+    struct stat st;
+    if( fstat( infd, &st ) == 0 && file_is_archive( st ) )
+      { show_file_error( filename, "File is the archive; not concatenated." );
+        close( infd ); continue; }
     if( !check_appendable( outfd, true ) )
       { show_error( "This does not look like an appendable tar.lz archive." );
         close( infd ); retval = 2; break; }
@@ -532,6 +561,10 @@ int encode( const std::string & archive_name, const Arg_parser & parser,
         return 2; }
     }
 
+  archive_namep = archive_name.size() ? archive_name.c_str() : "(stdout)";
+  if( !file_is_archive.init() )
+    { show_file_error( archive_namep, "Can't stat", errno ); return 1; }
+
   if( compressed )
     {
     encoder = LZ_compress_open( option_mapping[level].dictionary_size,
@@ -573,16 +606,18 @@ int encode( const std::string & archive_name, const Arg_parser & parser,
 
   if( !retval )			// write End-Of-Archive records
     {
-    uint8_t buf[header_size];
-    std::memset( buf, 0, header_size );
+    enum { bufsize = 2 * header_size };
+    uint8_t buf[bufsize];
+    std::memset( buf, 0, bufsize );
     if( encoder && cl_solid == 2 && !archive_write( 0, 0 ) )	// flush encoder
       { show_error( "Error flushing encoder", errno ); retval = 1; }
-    else if( !archive_write( buf, header_size ) ||
-             !archive_write( buf, header_size ) ||
+    else if( !archive_write( buf, bufsize ) ||
              ( encoder && !archive_write( 0, 0 ) ) )	// flush encoder
       { show_error( "Error writing end-of-archive blocks", errno );
         retval = 1; }
     }
+  if( encoder && LZ_compress_close( encoder ) < 0 )
+    { show_error( "LZ_compress_close failed." ); retval = 1; }
   if( close( outfd ) != 0 && !retval )
     { show_error( "Error closing archive", errno ); retval = 1; }
   if( retval && archive_name.size() && !append )
