@@ -42,6 +42,17 @@
 namespace {
 
 int gretval = 0;
+bool has_lz_ext;			// global var for archive_read
+
+void skip_warn( const bool reset = false )	// avoid duplicate warnings
+  {
+  static bool skipping = false;
+
+  if( reset ) skipping = false;
+  else if( !skipping )
+    { skipping = true; show_error( "Skipping to next header." ); }
+  }
+
 
 bool make_path( const std::string & name )
   {
@@ -72,42 +83,6 @@ bool make_path( const std::string & name )
   }
 
 
-// Returns in buf the first rd bytes of the second lzip member or
-// the first 512 bytes of the second tar member, and sets islz if lzip member
-bool skip_first_member( const int infd, uint8_t * const buf,
-                        int & rd, bool & islz )
-  {
-  while( true )
-    {
-    for( int i = 0; i < rd; ++i )
-      if( buf[i] == 'L' && (*(Lzip_header *)( buf + i )).verify_prefix( rd - i ) )
-        {
-        const int ts = rd - i;				// tail size
-        std::memmove( buf, buf + i, ts );
-        if( ts >= (int)sizeof lzip_magic )
-          { rd = ts; islz = true; return true; }
-        int rd2 = readblock( infd, buf + ts, header_size - ts );
-        if( rd2 != header_size - ts && errno )
-          { show_error( "Error reading archive", errno ); return false; }
-        if( ts + rd2 >= min_member_size &&
-            (*(Lzip_header *)buf).verify_magic() )
-          { rd = ts + rd2; islz = true; return true; }
-        std::memmove( buf, buf + ts, rd2 );
-        int rd3 = readblock( infd, buf + rd2, header_size - rd2 );
-        if( rd3 != header_size - rd2 && errno )
-          { show_error( "Error reading archive", errno ); return false; }
-        rd = rd2 + rd3; i = -1;
-        }
-    if( rd < header_size ) return false;		// eof
-    if( rd == header_size && verify_ustar_chksum( buf ) )
-      { islz = false; return true; }
-    rd = readblock( infd, buf, header_size );
-    if( rd != header_size && errno )
-      { show_error( "Error reading archive", errno ); return false; }
-    }
-  }
-
-
 inline bool block_is_zero( const uint8_t * const buf, const int size )
   {
   for( int i = 0; i < size; ++i ) if( buf[i] != 0 ) return false;
@@ -115,58 +90,80 @@ inline bool block_is_zero( const uint8_t * const buf, const int size )
   }
 
 
-bool archive_read( const int infd, uint8_t * const buf, const int size )
+// Return value: 0 = OK, 1 = damaged member, 2 = fatal error.
+// If sizep and error, return in *sizep the number of bytes read.
+// The first 6 bytes of the archive must be intact for islz to be meaningful.
+int archive_read( const int infd, uint8_t * const buf, const int size,
+                  int * const sizep = 0 )
   {
   static LZ_Decoder * decoder = 0;
-  static bool first_call = true;
   static bool at_eof = false;
+  static bool fatal = false;
+  static bool first_call = true;
 
+  if( sizep ) *sizep = 0;
+  if( fatal ) return 2;
   if( first_call )					// check format
     {
     first_call = false;
     if( size != header_size )
       internal_error( "size != header_size on first call." );
-    int rd = readblock( infd, buf, size );
+    const int rd = readblock( infd, buf, size );
+    if( sizep ) *sizep = rd;
     if( rd != size && errno )
-      { show_error( "Error reading archive", errno ); return false; }
-    bool islz =
-      ( rd >= min_member_size && (*(Lzip_header *)buf).verify_magic() );
+      { show_error( "Error reading archive", errno ); fatal = true; return 2; }
+    const Lzip_header & header = (*(const Lzip_header *)buf);
+    bool islz = ( rd >= min_member_size && header.verify_magic() &&
+                  isvalid_ds( header.dictionary_size() ) );
     const bool istar = ( rd == size && verify_ustar_chksum( buf ) );
     const bool iseof =
       ( !islz && !istar && rd == size && block_is_zero( buf, size ) );
-    if( !islz && !istar && !iseof )
+    if( !islz && !istar && !iseof )		// corrupt or invalid format
       {
-      show_error( "This does not look like a tar archive." );
-      show_error( "Skipping to next header." );
-//      std::fprintf( stderr, "%07o\n", ustar_chksum( buf ) );
-      gretval = 2;
-      if( !skip_first_member( infd, buf, rd, islz ) ) return false;
+      show_error( "This does not look like a POSIX tar archive." );
+      if( has_lz_ext ) islz = true; else return 1;
       }
-    if( !islz ) return true;				// uncompressed
+    if( !islz )						// uncompressed
+      { if( rd == size ) return 0; fatal = true; return 2; }
     decoder = LZ_decompress_open();			// compressed
     if( !decoder || LZ_decompress_errno( decoder ) != LZ_ok )
       { show_error( "Not enough memory." );
-        LZ_decompress_close( decoder ); return false; }
+        LZ_decompress_close( decoder ); fatal = true; return 2; }
     if( LZ_decompress_write( decoder, buf, rd ) != rd )
       internal_error( "library error (LZ_decompress_write)." );
-    if( !archive_read( infd, buf, size ) ) return false;
-    if( verify_ustar_chksum( buf ) || block_is_zero( buf, size ) ) return true;
-    show_error( "This does not look like a tar archive." );
-    show_error( "Skipping to next header." );
-    gretval = 2;
-    if( LZ_decompress_sync_to_member( decoder ) < 0 )
-      internal_error( "library error (LZ_decompress_sync_to_member)." );
+    const int res = archive_read( infd, buf, size, sizep );
+    if( res != 0 ) { if( res == 2 ) fatal = true; return res; }
+    if( verify_ustar_chksum( buf ) || block_is_zero( buf, size ) ) return 0;
+    show_error( "This does not look like a POSIX tar.lz archive." );
+    fatal = true; return 2;
     }
 
   if( !decoder )					// uncompressed
-    { if( readblock( infd, buf, size ) == size ) return true;
-      show_error( "Archive ends unexpectedly." ); return false; }
+    {
+    const int rd = readblock( infd, buf, size ); if( rd == size ) return 0;
+    if( sizep ) *sizep = rd;
+    show_error( "Archive ends unexpectedly." ); fatal = true; return 2;
+    }
   const int ibuf_size = 16384;
   uint8_t ibuf[ibuf_size];
   int sz = 0;
   while( sz < size )
     {
-    if( !at_eof && LZ_decompress_write_size( decoder ) > 0 )
+    const int rd = LZ_decompress_read( decoder, buf + sz, size - sz );
+    if( rd < 0 )
+      {
+      if( LZ_decompress_sync_to_member( decoder ) < 0 )
+        internal_error( "library error (LZ_decompress_sync_to_member)." );
+      skip_warn(); gretval = 2; return 1;
+      }
+    if( rd == 0 && LZ_decompress_finished( decoder ) == 1 )
+     { LZ_decompress_close( decoder );
+       show_error( "Archive ends unexpectedly." ); fatal = true; return 2; }
+    sz += rd; if( sizep ) *sizep = sz;
+    if( sz == size && LZ_decompress_finished( decoder ) == 1 &&
+        LZ_decompress_close( decoder ) < 0 )
+      { show_error( "LZ_decompress_close failed." ); fatal = true; return 2; }
+    if( sz < size && !at_eof && LZ_decompress_write_size( decoder ) > 0 )
       {
       const int rsize = std::min( ibuf_size, LZ_decompress_write_size( decoder ) );
       const int rd = readblock( infd, ibuf, rsize );
@@ -176,27 +173,12 @@ bool archive_read( const int infd, uint8_t * const buf, const int size )
         {
         at_eof = true; LZ_decompress_finish( decoder );
         if( errno )
-          { show_error( "Error reading archive", errno ); return false; }
+          { show_error( "Error reading archive", errno ); fatal = true;
+            return 2; }
         }
       }
-    const int rd = LZ_decompress_read( decoder, buf + sz, size - sz );
-    if( rd < 0 )
-      {
-      show_error( "Skipping to next header." );
-      gretval = 2;
-      if( LZ_decompress_sync_to_member( decoder ) < 0 )
-        internal_error( "library error (LZ_decompress_sync_to_member)." );
-      continue;
-      }
-    if( rd == 0 && LZ_decompress_finished( decoder ) == 1 )
-     { LZ_decompress_close( decoder );
-       show_error( "Archive ends unexpectedly." ); return false; }
-    sz += rd;
-    if( sz == size && LZ_decompress_finished( decoder ) == 1 &&
-        LZ_decompress_close( decoder ) < 0 )
-      { show_error( "LZ_decompress_close failed." ); return false; }
     }
-  return true;
+  return 0;
   }
 
 
@@ -291,7 +273,8 @@ int list_member( const int infd, const Extended & extended,
   while( rest > 0 )
     {
     const int rsize = ( rest >= bufsize ) ? bufsize : rest + padding;
-    if( !archive_read( infd, buf, rsize ) ) return 2;
+    const int ret = archive_read( infd, buf, rsize );
+    if( ret != 0 ) { if( ret == 2 ) return 2; else break; }
     if( rest < bufsize ) break;
     rest -= rsize;
     }
@@ -310,7 +293,7 @@ bool contains_dotdot( const char * const filename )
 
 
 int extract_member( const int infd, const Extended & extended,
-                    const Tar_header header )
+                    const Tar_header header, const bool keep_damaged )
   {
   const char * const filename = extended.path.c_str();
   if( contains_dotdot( filename ) )
@@ -407,9 +390,19 @@ int extract_member( const int infd, const Extended & extended,
   while( rest > 0 )
     {
     const int rsize = ( rest >= bufsize ) ? bufsize : rest + padding;
-    if( !archive_read( infd, buf, rsize ) )
-      { if( outfd >= 0 ) { close( outfd ); std::remove( filename ); }
-        return 2; }
+    int rd;
+    const int ret = archive_read( infd, buf, rsize, &rd );
+    if( ret != 0 )
+      {
+      if( outfd >= 0 )
+        {
+        if( keep_damaged )
+          { writeblock( outfd, buf, std::min( rest, (unsigned long long)rd ) );
+            close( outfd ); }
+        else { close( outfd ); std::remove( filename ); }
+        }
+      if( ret == 2 ) return 2; else return 0;
+      }
     const int wsize = ( rest >= bufsize ) ? bufsize : rest;
     if( outfd >= 0 && writeblock( outfd, buf, wsize ) != wsize )
       { show_file_error( filename, "Error writing file", errno ); return 2; }
@@ -478,7 +471,7 @@ bool Extended::parse( const int infd, const Tar_header header,
   if( bufsize == 0 || edsize == 0 || edsize >= 1ULL << 33 )
     return false;				// overflow or no extended data
   char * const buf = new char[bufsize];		// extended records buffer
-  if( !archive_read( infd, (uint8_t *)buf, bufsize ) ) goto error;
+  if( archive_read( infd, (uint8_t *)buf, bufsize ) != 0 ) goto error;
   for( unsigned long long pos = 0; pos < edsize; )	// parse records
     {
     char * tail;
@@ -530,8 +523,8 @@ error:
 
 
 int decode( const std::string & archive_name, const Arg_parser & parser,
-            const int filenames, const bool listing, const bool missing_crc,
-            const bool permissive )
+            const int filenames, const bool keep_damaged, const bool listing,
+            const bool missing_crc, const bool permissive )
   {
   const int infd = archive_name.size() ?
                    open_instream( archive_name ) : STDIN_FILENO;
@@ -552,23 +545,25 @@ int decode( const std::string & archive_name, const Arg_parser & parser,
     if( !code ) name_pending[i] = true;
     }
 
+  has_lz_ext =
+    ( archive_name.size() > 3 &&
+      archive_name.compare( archive_name.size() - 3, 3, ".lz" ) == 0 ) ||
+    ( archive_name.size() > 4 &&
+      archive_name.compare( archive_name.size() - 4, 4, ".tlz" ) == 0 );
   Extended extended;		// metadata from extended records
   int retval = 0;
   bool prev_extended = false;	// prev header was extended
-  bool skipping = false;
   while( true )			// process one member per iteration
     {
     uint8_t buf[header_size];
-    if( !archive_read( infd, buf, header_size ) ) return 2;
-    if( !verify_ustar_chksum( buf ) )
+    const int ret = archive_read( infd, buf, header_size );
+    if( ret == 2 ) return 2;
+    if( ret != 0 || !verify_ustar_chksum( buf ) )
       {
-      if( block_is_zero( buf, header_size ) ) break;
-      gretval = 2;
-      if( !skipping )
-        { skipping = true; show_error( "Skipping to next header." ); }
-      continue;
+      if( ret == 0 && block_is_zero( buf, header_size ) ) break;	// EOF
+      skip_warn(); gretval = 2; continue;
       }
-    skipping = false;
+    skip_warn( true );		// reset warning
 
     const char * const header = (const char *)buf;
     const Typeflag typeflag = (Typeflag)header[typeflag_o];
@@ -626,9 +621,11 @@ int decode( const std::string & archive_name, const Arg_parser & parser,
     if( listing || skip )
       retval = list_member( infd, extended, header, skip );
     else
-      retval = extract_member( infd, extended, header );
+      retval = extract_member( infd, extended, header, keep_damaged );
     extended.reset();
-    if( retval ) return retval;
+    if( retval )
+      { show_error( "Error is not recoverable: exiting now." );
+        return retval; }
     }
 
   for( int i = 0; i < parser.arguments(); ++i )
