@@ -40,6 +40,7 @@
 #if defined(__OS2__)
 #include <io.h>
 #endif
+#include <lzlib.h>
 
 #include "arg_parser.h"
 #include "tarlz.h"
@@ -83,9 +84,11 @@ void show_help()
                "  -0 .. -9                   set compression level [default 6]\n"
                "      --asolid               create solidly compressed appendable archive\n"
                "      --dsolid               create per-directory compressed archive\n"
+               "      --no-solid             create per-file compressed archive (default)\n"
                "      --solid                create solidly compressed archive\n"
-               "      --group=<group>        use <group> name/id for added files\n"
-               "      --owner=<owner>        use <owner> name/id for added files\n"
+               "      --group=<group>        use <group> name/ID for files added\n"
+               "      --owner=<owner>        use <owner> name/ID for files added\n"
+               "      --keep-damaged         don't delete partially extracted files\n"
                "      --missing-crc          exit with error status if missing extended CRC\n"
 //               "      --permissive           allow repeated extended headers and records\n"
                "      --uncompressed         don't compress the created archive\n"
@@ -102,6 +105,7 @@ void show_version()
   {
   std::printf( "%s %s\n", program_name, PROGVERSION );
   std::printf( "Copyright (C) %s Antonio Diaz Diaz.\n", program_year );
+  std::printf( "Using lzlib %s\n", LZ_version() );
   std::printf( "License GPLv2+: GNU GPL version 2 or later <http://gnu.org/licenses/gpl.html>\n"
                "This is free software: you are free to change and redistribute it.\n"
                "There is NO WARRANTY, to the extent permitted by law.\n" );
@@ -283,45 +287,52 @@ int main( const int argc, const char * const argv[] )
   std::string archive_name;
   Mode program_mode = m_none;
   int level = 6;		// compression level, < 0 = uncompressed
+  bool keep_damaged = false;
   bool missing_crc = false;
   bool permissive = false;
   invocation_name = argv[0];
 
-  enum { opt_aso = 256, opt_crc, opt_dso, opt_grp, opt_own, opt_per, opt_sol,
-         opt_un };
+  if( LZ_version()[0] < '1' )
+    { show_error( "Bad library version. At least lzlib 1.0 is required." );
+      return 1; }
+
+  enum { opt_aso = 256, opt_crc, opt_dso, opt_grp, opt_kd, opt_nso, opt_own,
+         opt_per, opt_sol, opt_un };
   const Arg_parser::Option options[] =
     {
-    { '0',  0,                Arg_parser::no  },
-    { '1',  0,                Arg_parser::no  },
-    { '2',  0,                Arg_parser::no  },
-    { '3',  0,                Arg_parser::no  },
-    { '4',  0,                Arg_parser::no  },
-    { '5',  0,                Arg_parser::no  },
-    { '6',  0,                Arg_parser::no  },
-    { '7',  0,                Arg_parser::no  },
-    { '8',  0,                Arg_parser::no  },
-    { '9',  0,                Arg_parser::no  },
-    { 'A', "concatenate",     Arg_parser::no  },
-    { 'c', "create",          Arg_parser::no  },
-    { 'C', "directory",       Arg_parser::yes },
-    { 'f', "file",            Arg_parser::yes },
-    { 'h', "help",            Arg_parser::no  },
-    { 'H', "format",          Arg_parser::yes },
-    { 'q', "quiet",           Arg_parser::no  },
-    { 'r', "append",          Arg_parser::no  },
-    { 't', "list",            Arg_parser::no  },
-    { 'v', "verbose",         Arg_parser::no  },
-    { 'V', "version",         Arg_parser::no  },
-    { 'x', "extract",         Arg_parser::no  },
-    { opt_aso, "asolid",      Arg_parser::no  },
-    { opt_dso, "dsolid",      Arg_parser::no  },
-    { opt_grp, "group",       Arg_parser::yes },
-    { opt_crc, "missing-crc", Arg_parser::no  },
-    { opt_own, "owner",       Arg_parser::yes },
-    { opt_per, "permissive",  Arg_parser::no  },
-    { opt_sol, "solid",       Arg_parser::no  },
-    { opt_un, "uncompressed", Arg_parser::no  },
-    {  0 ,  0,                Arg_parser::no  } };
+    { '0',  0,                 Arg_parser::no  },
+    { '1',  0,                 Arg_parser::no  },
+    { '2',  0,                 Arg_parser::no  },
+    { '3',  0,                 Arg_parser::no  },
+    { '4',  0,                 Arg_parser::no  },
+    { '5',  0,                 Arg_parser::no  },
+    { '6',  0,                 Arg_parser::no  },
+    { '7',  0,                 Arg_parser::no  },
+    { '8',  0,                 Arg_parser::no  },
+    { '9',  0,                 Arg_parser::no  },
+    { 'A', "concatenate",      Arg_parser::no  },
+    { 'c', "create",           Arg_parser::no  },
+    { 'C', "directory",        Arg_parser::yes },
+    { 'f', "file",             Arg_parser::yes },
+    { 'h', "help",             Arg_parser::no  },
+    { 'H', "format",           Arg_parser::yes },
+    { 'q', "quiet",            Arg_parser::no  },
+    { 'r', "append",           Arg_parser::no  },
+    { 't', "list",             Arg_parser::no  },
+    { 'v', "verbose",          Arg_parser::no  },
+    { 'V', "version",          Arg_parser::no  },
+    { 'x', "extract",          Arg_parser::no  },
+    { opt_aso, "asolid",       Arg_parser::no  },
+    { opt_dso, "dsolid",       Arg_parser::no  },
+    { opt_grp, "group",        Arg_parser::yes },
+    { opt_kd,  "keep-damaged", Arg_parser::no  },
+    { opt_crc, "missing-crc",  Arg_parser::no  },
+    { opt_nso, "no-solid",     Arg_parser::no  },
+    { opt_own, "owner",        Arg_parser::yes },
+    { opt_per, "permissive",   Arg_parser::no  },
+    { opt_sol, "solid",        Arg_parser::no  },
+    { opt_un,  "uncompressed", Arg_parser::no  },
+    {  0 ,  0,                 Arg_parser::no  } };
 
   const Arg_parser parser( argc, argv, options, true );
   if( parser.error().size() )				// bad option
@@ -355,10 +366,12 @@ int main( const int argc, const char * const argv[] )
       case opt_crc: missing_crc = true; break;
       case opt_dso: cl_solid = 1; break;
       case opt_grp: set_group( arg ); break;
+      case opt_kd:  keep_damaged = true; break;
+      case opt_nso: cl_solid = 0; break;
       case opt_own: set_owner( arg ); break;
       case opt_per: permissive = true; break;
       case opt_sol: cl_solid = 3; break;
-      case opt_un: level = -1; break;
+      case opt_un:  level = -1; break;
       default : internal_error( "uncaught option" );
       }
     } // end process options
@@ -377,6 +390,7 @@ int main( const int argc, const char * const argv[] )
     case m_concatenate: return concatenate( archive_name, parser, filenames );
     case m_extract:
     case m_list:   return decode( archive_name, parser, filenames,
-                            program_mode == m_list, missing_crc, permissive );
+                                  keep_damaged, program_mode == m_list,
+                                  missing_crc, permissive );
     }
   }
