@@ -118,8 +118,8 @@ int encode( const std::string & archive_name, const Arg_parser & parser,
 
 // defined in extract.cc
 int decode( const std::string & archive_name, const Arg_parser & parser,
-            const int filenames, const bool listing, const bool missing_crc,
-            const bool permissive );
+            const int filenames, const bool keep_damaged, const bool listing,
+            const bool missing_crc, const bool permissive );
 
 // defined in main.cc
 extern int verbosity;
