@@ -32,11 +32,15 @@ in="${testdir}"/test.txt
 in_lz="${testdir}"/test.txt.lz
 in_tar="${testdir}"/test.txt.tar
 in_tar_lz="${testdir}"/test.txt.tar.lz
+inbad1="${testdir}"/test_bad1.txt
+inbad2="${testdir}"/test_bad2.txt
 test3="${testdir}"/test3.tar
 test3_lz="${testdir}"/test3.tar.lz
 test3dir_lz="${testdir}"/test3_dir.tar.lz
 t155="${testdir}"/t155.tar
 t155_lz="${testdir}"/t155.tar.lz
+tlzit1="${testdir}"/tlz_in_tar1.tar
+tlzit2="${testdir}"/tlz_in_tar2.tar
 bad1="${testdir}"/test3_bad1.tar
 bad2="${testdir}"/test3_bad2.tar
 bad3="${testdir}"/test3_bad3.tar
@@ -50,10 +54,16 @@ bad5_lz="${testdir}"/test3_bad5.tar.lz
 bad6_lz="${testdir}"/test3_bad6.tar.lz
 eof_lz="${testdir}"/eof.tar.lz
 fail=0
+lwarn=0
 test_failed() { fail=1 ; printf " $1" ; [ -z "$2" ] || printf "($2)" ; }
+lzlib_1_11() { [ ${lwarn} = 0 ] &&
+	printf "\nwarning: testing --keep-damaged requires lzlib-1.11-rc2 or newer\n$1"
+	lwarn=1 ; }
 
 # Description of test files for tarlz:
 # t155.tar[.lz]      directory + file + link + eof, all with 155 char names
+# test_bad1.tar.lz:  truncated at offset 6000 (of 7495)
+# test_bad2.tar.lz:  byte at offset 6000 changed from 0x56 to 0x46
 # test3.tar:         3 members (foo bar baz) + 2 zeroed 512-byte blocks
 # test3_dir.tar.lz:  like test3.tar.lz but members dir/foo dir/bar dir/baz
 # test3_bad1.tar:    byte at offset  259 changed from 't' to '0' (magic)
@@ -64,153 +74,170 @@ test_failed() { fail=1 ; printf " $1" ; [ -z "$2" ] || printf "($2)" ; }
 # test3_bad5.tar:    510 zeros + "LZ" prepended to test3.tar (bogus lz header)
 # test3_bad1.tar.lz: byte at offset    2 changed from 'I' to 'i' (magic)
 # test3_bad2.tar.lz: byte at offset   49 changed from 0x49 to 0x69 (mid stream)
-# test3_bad3.tar.lz: byte at offset  149 changed from 0x2D to 0x3D (mid stream)
+# test3_bad3.tar.lz: byte at offset  176 changed from 0x7D to 0x6D (mid stream)
 # test3_bad4.tar.lz: combined damage of test3_bad2.tar.lz and test3_bad3.tar.lz
 # test3_bad5.tar.lz: [71-134] --> zeroed (first trailer + seconf header)
 # test3_bad6.tar.lz: 510 zeros prepended to test3.tar.lz (header in two blocks)
+# tlz_in_tar1.tar:   1 member (test3.tar.lz) first magic damaged
+# tlz_in_tar2.tar:   2 members (foo test3.tar.lz) first magic damaged
 
 printf "testing tarlz-%s..." "$2"
 
-"${TARLZ}" -qtf ${in}
+"${TARLZ}" -q -tf "${in}"
 [ $? = 2 ] || test_failed $LINENO
-"${TARLZ}" -qtf ${in_lz}
+"${TARLZ}" -q -tf "${in_lz}"
 [ $? = 2 ] || test_failed $LINENO
-"${TARLZ}" -qtf nx_file
+"${TARLZ}" -q -tf nx_file
 [ $? = 1 ] || test_failed $LINENO
 "${TARLZ}" -tf 2> /dev/null
 [ $? = 1 ] || test_failed $LINENO
-"${TARLZ}" -qcf out.tar.lz
+"${TARLZ}" -q -cf out.tar.lz
 [ $? = 1 ] || test_failed $LINENO
 [ ! -e out.tar.lz ] || test_failed $LINENO
 "${TARLZ}" -rf out.tar.lz || test_failed $LINENO
 [ ! -e out.tar.lz ] || test_failed $LINENO
-"${TARLZ}" -qrf - ${in}
+"${TARLZ}" -q -rf - "${in}"
 [ $? = 1 ] || test_failed $LINENO
 [ ! -e - ] || test_failed $LINENO
-"${TARLZ}" -qr ${in}
+"${TARLZ}" -q -r "${in}"
 [ $? = 1 ] || test_failed $LINENO
-"${TARLZ}" --uncompressed -qrf out.tar ${in}
+"${TARLZ}" --uncompressed -q -rf out.tar "${in}"
 [ $? = 1 ] || test_failed $LINENO
 [ ! -e out.tar ] || test_failed $LINENO
-cat ${test3_lz} > test.tar.lz || framework_failure
-"${TARLZ}" --uncompressed -qrf test.tar.lz ${in}
+cat "${test3_lz}" > test.tar.lz || framework_failure
+"${TARLZ}" --uncompressed -q -rf test.tar.lz "${in}"
 [ $? = 1 ] || test_failed $LINENO
-cmp ${test3_lz} test.tar.lz || test_failed $LINENO
+cmp "${test3_lz}" test.tar.lz || test_failed $LINENO
 rm -f test.tar.lz || framework_failure
-cat ${test3} > test.tar || framework_failure
-"${TARLZ}" -qrf test.tar ${in}
+cat "${test3}" > test.tar || framework_failure
+"${TARLZ}" -q -rf test.tar "${in}"
 [ $? = 2 ] || test_failed $LINENO
-cmp ${test3} test.tar || test_failed $LINENO
+cmp "${test3}" test.tar || test_failed $LINENO
 rm -f test.tar || framework_failure
-"${TARLZ}" -qc ${in} nx_file > /dev/null
+"${TARLZ}" -q -c "${in}" nx_file > /dev/null
 [ $? = 1 ] || test_failed $LINENO
-"${TARLZ}" -qc -C nx_dir ${in}
+"${TARLZ}" -q -c -C nx_dir "${in}"
 [ $? = 1 ] || test_failed $LINENO
-"${TARLZ}" -qx -C nx_dir ${test3_lz}
+"${TARLZ}" -q -x -C nx_dir "${test3_lz}"
 [ $? = 1 ] || test_failed $LINENO
-"${TARLZ}" -qcr
+"${TARLZ}" -q -cr
 [ $? = 1 ] || test_failed $LINENO
-"${TARLZ}" -qct
+"${TARLZ}" -q -ct
 [ $? = 1 ] || test_failed $LINENO
-"${TARLZ}" -qcx
+"${TARLZ}" -q -cx
 [ $? = 1 ] || test_failed $LINENO
-"${TARLZ}" -qtx
+"${TARLZ}" -q -tx
 [ $? = 1 ] || test_failed $LINENO
-"${TARLZ}" -qctx
+"${TARLZ}" -q -ctx
 [ $? = 1 ] || test_failed $LINENO
 "${TARLZ}" --help > /dev/null || test_failed $LINENO
 "${TARLZ}" -V > /dev/null || test_failed $LINENO
-"${TARLZ}" --bad_option -tf ${test3_lz} 2> /dev/null
+"${TARLZ}" --bad_option -tf "${test3_lz}" 2> /dev/null
 [ $? = 1 ] || test_failed $LINENO
 "${TARLZ}" -tf 2> /dev/null
 [ $? = 1 ] || test_failed $LINENO
-"${TARLZ}" --owner=invalid_oner_name -tf ${test3_lz} 2> /dev/null
+"${TARLZ}" --owner=invalid_oner_name -tf "${test3_lz}" 2> /dev/null
 [ $? = 1 ] || test_failed $LINENO
-"${TARLZ}" --group=invalid_goup_name -tf ${test3_lz} 2> /dev/null
+"${TARLZ}" --group=invalid_goup_name -tf "${test3_lz}" 2> /dev/null
 [ $? = 1 ] || test_failed $LINENO
 
-"${TARLZ}" -tf ${eof_lz} --missing-crc || test_failed $LINENO
-"${TARLZ}" -xf ${eof_lz} --missing-crc || test_failed $LINENO
-"${TARLZ}" -tf ${in_tar_lz} --missing-crc > /dev/null || test_failed $LINENO
-"${TARLZ}" -xf ${in_tar_lz} --missing-crc || test_failed $LINENO
-cmp ${in} test.txt || test_failed $LINENO
+# test --list and --extract
+"${TARLZ}" -tf "${eof_lz}" --missing-crc || test_failed $LINENO
+"${TARLZ}" -xf "${eof_lz}" --missing-crc || test_failed $LINENO
+"${TARLZ}" -tf "${in_tar_lz}" --missing-crc > /dev/null || test_failed $LINENO
+"${TARLZ}" -xf "${in_tar_lz}" --missing-crc || test_failed $LINENO
+cmp "${in}" test.txt || test_failed $LINENO
 rm -f test.txt || framework_failure
-"${TARLZ}" -C nx_dir -tf ${in_tar} > /dev/null || test_failed $LINENO
-"${TARLZ}" -xf ${in_tar} --missing-crc || test_failed $LINENO
-cmp ${in} test.txt || test_failed $LINENO
+"${TARLZ}" -C nx_dir -tf "${in_tar}" > /dev/null || test_failed $LINENO
+"${TARLZ}" -xf "${in_tar}" --missing-crc || test_failed $LINENO
+cmp "${in}" test.txt || test_failed $LINENO
 rm -f test.txt || framework_failure
 
+printf "foo\n" > cfoo || framework_failure
+printf "bar\n" > cbar || framework_failure
+printf "baz\n" > cbaz || framework_failure
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -xf ${test3_lz} --missing-crc || test_failed $LINENO
-[ -e foo ] || test_failed $LINENO
-[ -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+"${TARLZ}" -xf "${test3_lz}" --missing-crc || test_failed $LINENO
+cmp cfoo foo || test_failed $LINENO
+cmp cbar bar || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -xf ${test3} --missing-crc || test_failed $LINENO
-[ -e foo ] || test_failed $LINENO
-[ -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+"${TARLZ}" -xf "${test3}" --missing-crc || test_failed $LINENO
+cmp cfoo foo || test_failed $LINENO
+cmp cbar bar || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${test3dir_lz} --missing-crc || test_failed $LINENO
-[ -e dir/foo ] || test_failed $LINENO
-[ -e dir/bar ] || test_failed $LINENO
-[ -e dir/baz ] || test_failed $LINENO
+"${TARLZ}" -q -xf "${test3dir_lz}" --missing-crc || test_failed $LINENO
+cmp cfoo dir/foo || test_failed $LINENO
+cmp cbar dir/bar || test_failed $LINENO
+cmp cbaz dir/baz || test_failed $LINENO
 rm -rf dir || framework_failure
 
-cat ${in_tar_lz} > out.tar.lz || framework_failure
-"${TARLZ}" -Af out.tar.lz ${test3_lz} || test_failed $LINENO
+# test --concatenate
+cat "${in_tar_lz}" > out.tar.lz || framework_failure
+"${TARLZ}" -Af out.tar.lz "${test3_lz}" || test_failed $LINENO
 "${TARLZ}" -xf out.tar.lz || test_failed $LINENO
-[ -e test.txt ] || test_failed $LINENO
-[ -e foo ] || test_failed $LINENO
-[ -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+cmp "${in}" test.txt || test_failed $LINENO
+cmp cfoo foo || test_failed $LINENO
+cmp cbar bar || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 touch aout.tar.lz || framework_failure		# concatenate to empty file
-"${TARLZ}" -Af aout.tar.lz ${in_tar_lz} ${test3_lz} || test_failed $LINENO
+"${TARLZ}" -Af aout.tar.lz "${in_tar_lz}" "${test3_lz}" || test_failed $LINENO
 cmp out.tar.lz aout.tar.lz || test_failed $LINENO
-cat ${in_tar_lz} > aout.tar.lz || framework_failure
-"${TARLZ}" -Aqf aout.tar.lz ${test3_lz} ${test3}
+cat "${in_tar_lz}" > aout.tar.lz || framework_failure
+"${TARLZ}" -Aqf aout.tar.lz "${test3_lz}" "${test3}"
 [ $? = 2 ] || test_failed $LINENO
 cmp out.tar.lz aout.tar.lz || test_failed $LINENO
 rm -f test.txt foo bar baz out.tar.lz aout.tar.lz || framework_failure
 
-cat ${in} > test.txt || framework_failure
+# test --create
+cat "${in}" > test.txt || framework_failure
 "${TARLZ}" -0 -cf out.tar.lz test.txt || test_failed $LINENO
 rm -f test.txt || framework_failure
 "${TARLZ}" -xf out.tar.lz --missing-crc || test_failed $LINENO
-cmp ${in} test.txt || test_failed $LINENO
-cat ${in} > test.txt || framework_failure
+cmp "${in}" test.txt || test_failed $LINENO
+cat "${in}" > test.txt || framework_failure
 "${TARLZ}" --uncompressed -cf out.tar test.txt || test_failed $LINENO
 rm -f test.txt || framework_failure
 "${TARLZ}" -xf out.tar --missing-crc || test_failed $LINENO
-cmp ${in} test.txt || test_failed $LINENO
+cmp "${in}" test.txt || test_failed $LINENO
 rm -f test.txt out.tar out.tar.lz || framework_failure
 
-printf "foo" > foo || framework_failure
+cat cfoo > foo || framework_failure
 rm -f bar || framework_failure
-printf "baz" > baz || framework_failure
-"${TARLZ}" -qcf out.tar.lz foo bar baz
+cat cbaz > baz || framework_failure
+"${TARLZ}" -q -cf out.tar.lz foo bar baz
 [ $? = 1 ] || test_failed $LINENO
 rm -f foo bar baz || framework_failure
 "${TARLZ}" -xf out.tar.lz --missing-crc || test_failed $LINENO
-[ -e foo ] || test_failed $LINENO
+cmp cfoo foo || test_failed $LINENO
 [ ! -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf out.tar.lz bar
+"${TARLZ}" -q -xf out.tar.lz bar
 [ $? = 1 ] || test_failed $LINENO
 [ ! -e foo ] || test_failed $LINENO
 [ ! -e bar ] || test_failed $LINENO
 [ ! -e baz ] || test_failed $LINENO
 rm -f out.tar.lz || framework_failure
 
-printf "foo" > foo || framework_failure
-printf "bar" > bar || framework_failure
-printf "baz" > baz || framework_failure
+cat cfoo > foo || framework_failure
+cat cbar > bar || framework_failure
+cat cbaz > baz || framework_failure
 "${TARLZ}" -0 -cf out.tar.lz foo bar baz || test_failed $LINENO
+"${TARLZ}" -0 -q -cf aout.tar.lz foo bar aout.tar.lz baz || test_failed $LINENO
+cmp out.tar.lz aout.tar.lz || test_failed $LINENO
+"${TARLZ}" -q -Af aout.tar.lz aout.tar.lz || test_failed $LINENO
+cmp out.tar.lz aout.tar.lz || test_failed $LINENO
+"${TARLZ}" -q -rf aout.tar.lz aout.tar.lz || test_failed $LINENO
+cmp out.tar.lz aout.tar.lz || test_failed $LINENO
+rm -f aout.tar.lz || framework_failure
+
+# test --append
 "${TARLZ}" --dsolid -0 -cf aout.tar.lz foo bar baz || test_failed $LINENO
 cmp out.tar.lz aout.tar.lz || test_failed $LINENO
 rm -f aout.tar.lz || framework_failure
-"${TARLZ}" -0 -qcf aout.tar.lz foo/ ./bar ./baz/ || test_failed $LINENO
+"${TARLZ}" -0 -q -cf aout.tar.lz foo/ ./bar ./baz/ || test_failed $LINENO
 cmp out.tar.lz aout.tar.lz || test_failed $LINENO
 rm -f aout.tar.lz || framework_failure
 "${TARLZ}" -0 -cf aout.tar.lz foo || test_failed $LINENO
@@ -224,10 +251,10 @@ cmp out.tar.lz aout.tar.lz || test_failed $LINENO
 cmp out.tar.lz aout.tar.lz || test_failed $LINENO
 "${TARLZ}" -0 -rf aout.tar.lz -C nx_dir || test_failed $LINENO
 cmp out.tar.lz aout.tar.lz || test_failed $LINENO
-"${TARLZ}" -0 -qrf aout.tar.lz nx_file
+"${TARLZ}" -0 -q -rf aout.tar.lz nx_file
 [ $? = 1 ] || test_failed $LINENO
 cmp out.tar.lz aout.tar.lz || test_failed $LINENO
-cat ${eof_lz} > aout.tar.lz || framework_failure	# append to empty archive
+cat "${eof_lz}" > aout.tar.lz || framework_failure	# append to empty archive
 "${TARLZ}" -0 -rf aout.tar.lz foo bar baz || test_failed $LINENO
 cmp out.tar.lz aout.tar.lz || test_failed $LINENO
 mv -f foo foo.orig || framework_failure
@@ -282,16 +309,17 @@ rm -rf dir1 || framework_failure
 rm -f out.tar.lz aout.tar.lz || framework_failure
 
 # append to solid archive
-printf "foo" > foo || framework_failure
-printf "bar" > bar || framework_failure
-printf "baz" > baz || framework_failure
+cat cfoo > foo || framework_failure
+cat cbar > bar || framework_failure
+cat cbaz > baz || framework_failure
 "${TARLZ}" --solid -0 -cf out.tar.lz foo || test_failed $LINENO
+cat out.tar.lz > aout.tar.lz || framework_failure
 for i in --asolid --dsolid --solid -0 ; do
-	"${TARLZ}" $i -qrf out.tar.lz bar baz
+	"${TARLZ}" $i -q -rf out.tar.lz bar baz
 	[ $? = 2 ] || test_failed $LINENO $i
-	[ -e out.tar.lz ] || test_failed $LINENO $i
+	cmp out.tar.lz aout.tar.lz || test_failed $LINENO $i
 done
-rm -f out.tar.lz || framework_failure
+rm -f out.tar.lz aout.tar.lz || framework_failure
 for i in --asolid --dsolid -0 ; do
 	for j in --asolid --dsolid --solid -0 ; do
 		"${TARLZ}" $i -0 -cf out.tar.lz foo ||
@@ -308,6 +336,7 @@ for i in --asolid --dsolid -0 ; do
 done
 rm -f foo foo.orig bar bar.orig baz baz.orig || framework_failure
 
+# test directories and links
 mkdir dir1 || framework_failure
 "${TARLZ}" -0 -cf out.tar dir1 || test_failed $LINENO
 rmdir dir1 || framework_failure
@@ -332,55 +361,44 @@ if ln dummy_file dummy_link 2> /dev/null &&
 	mkdir dir1 || framework_failure
 	mkdir dir1/dir2 || framework_failure
 	mkdir dir1/dir2/dir3 || framework_failure
-	cat ${in} > dir1/dir2/dir3/in || framework_failure
-	ln dir1/dir2/dir3/in dir1/dir2/dir3/${name_100} || framework_failure
-	ln dir1/dir2/dir3/in ${path_100} || framework_failure
-	ln dir1/dir2/dir3/in ${path_106} || framework_failure
+	cat "${in}" > dir1/dir2/dir3/in || framework_failure
+	ln dir1/dir2/dir3/in dir1/dir2/dir3/"${name_100}" || framework_failure
+	ln dir1/dir2/dir3/in "${path_100}" || framework_failure
+	ln dir1/dir2/dir3/in "${path_106}" || framework_failure
 	ln -s in dir1/dir2/dir3/link || framework_failure
-	ln -s ${name_100} dir1/dir2/dir3/link_100 || framework_failure
+	ln -s "${name_100}" dir1/dir2/dir3/link_100 || framework_failure
 	"${TARLZ}" -0 -cf out.tar dir1 || test_failed $LINENO
 	rm -rf dir1 || framework_failure
 	"${TARLZ}" -xf out.tar || test_failed $LINENO
-	cmp ${in} dir1/dir2/dir3/in || test_failed $LINENO
-	cmp ${in} dir1/dir2/dir3/${name_100} || test_failed $LINENO
-	cmp ${in} ${path_100} || test_failed $LINENO
-	cmp ${in} ${path_106} || test_failed $LINENO
-	cmp ${in} dir1/dir2/dir3/link || test_failed $LINENO
-	cmp ${in} dir1/dir2/dir3/link_100 || test_failed $LINENO
+	cmp "${in}" dir1/dir2/dir3/in || test_failed $LINENO
+	cmp "${in}" dir1/dir2/dir3/"${name_100}" || test_failed $LINENO
+	cmp "${in}" "${path_100}" || test_failed $LINENO
+	cmp "${in}" "${path_106}" || test_failed $LINENO
+	cmp "${in}" dir1/dir2/dir3/link || test_failed $LINENO
+	cmp "${in}" dir1/dir2/dir3/link_100 || test_failed $LINENO
 	rm -f dir1/dir2/dir3/in || framework_failure
-	cmp ${in} dir1/dir2/dir3/link 2> /dev/null && test_failed $LINENO
-	cmp ${in} dir1/dir2/dir3/link_100 || test_failed $LINENO
+	cmp "${in}" dir1/dir2/dir3/link 2> /dev/null && test_failed $LINENO
+	cmp "${in}" dir1/dir2/dir3/link_100 || test_failed $LINENO
 	"${TARLZ}" -xf out.tar || test_failed $LINENO
 	rm -f out.tar || framework_failure
-	cmp ${in} dir1/dir2/dir3/in || test_failed $LINENO
-	cmp ${in} dir1/dir2/dir3/link || test_failed $LINENO
-	"${TARLZ}" -0 -qc ../tmp/dir1 > /dev/null || test_failed $LINENO
+	cmp "${in}" dir1/dir2/dir3/in || test_failed $LINENO
+	cmp "${in}" dir1/dir2/dir3/link || test_failed $LINENO
+	"${TARLZ}" -0 -q -c ../tmp/dir1 > /dev/null || test_failed $LINENO
 	rm -rf dir1 || framework_failure
 else
 	printf "\nwarning: skipping link test: 'ln' does not work on your system."
 fi
 rm -f dummy_slink dummy_link dummy_file || framework_failure
 
-"${TARLZ}" -qxf "${testdir}"/dotdot1.tar.lz || test_failed $LINENO
-[ ! -e ../dir ] || test_failed $LINENO
-"${TARLZ}" -qxf "${testdir}"/dotdot2.tar.lz || test_failed $LINENO
-[ ! -e ../dir ] || test_failed $LINENO
-"${TARLZ}" -qxf "${testdir}"/dotdot3.tar.lz || test_failed $LINENO
-[ ! -e dir ] || test_failed $LINENO
-"${TARLZ}" -qxf "${testdir}"/dotdot4.tar.lz || test_failed $LINENO
-[ ! -e dir ] || test_failed $LINENO
-"${TARLZ}" -qxf "${testdir}"/dotdot5.tar.lz || test_failed $LINENO
-[ ! -e dir ] || test_failed $LINENO
-
 printf "\ntesting long names..."
 
-"${TARLZ}" -qtf ${t155} || test_failed $LINENO
-"${TARLZ}" -qtf ${t155_lz} || test_failed $LINENO
+"${TARLZ}" -q -tf "${t155}" || test_failed $LINENO
+"${TARLZ}" -q -tf "${t155_lz}" || test_failed $LINENO
 if [ "${ln_works}" = yes ] ; then
 	mkdir dir1 || framework_failure
-	"${TARLZ}" -C dir1 -xf ${t155} || test_failed $LINENO
+	"${TARLZ}" -C dir1 -xf "${t155}" || test_failed $LINENO
 	mkdir dir2 || framework_failure
-	"${TARLZ}" -C dir2 -xf ${t155_lz} || test_failed $LINENO
+	"${TARLZ}" -C dir2 -xf "${t155_lz}" || test_failed $LINENO
 	diff -r dir1 dir2 || test_failed $LINENO
 	rmdir dir2 2> /dev/null && test_failed $LINENO
 	rmdir dir1 2> /dev/null && test_failed $LINENO
@@ -389,81 +407,153 @@ fi
 
 printf "\ntesting bad input..."
 
-dd if=${in_tar} of=truncated.tar bs=1000 count=1 2> /dev/null
-"${TARLZ}" -qtf truncated.tar > /dev/null
+"${TARLZ}" -q -xf "${testdir}"/dotdot1.tar.lz || test_failed $LINENO
+[ ! -e ../dir ] || test_failed $LINENO
+"${TARLZ}" -q -xf "${testdir}"/dotdot2.tar.lz || test_failed $LINENO
+[ ! -e ../dir ] || test_failed $LINENO
+"${TARLZ}" -q -xf "${testdir}"/dotdot3.tar.lz || test_failed $LINENO
+[ ! -e dir ] || test_failed $LINENO
+"${TARLZ}" -q -xf "${testdir}"/dotdot4.tar.lz || test_failed $LINENO
+[ ! -e dir ] || test_failed $LINENO
+"${TARLZ}" -q -xf "${testdir}"/dotdot5.tar.lz || test_failed $LINENO
+[ ! -e dir ] || test_failed $LINENO
+
+dd if="${in_tar}" of=truncated.tar bs=1000 count=1 2> /dev/null
+"${TARLZ}" -q -tf truncated.tar > /dev/null
 [ $? = 2 ] || test_failed $LINENO
-"${TARLZ}" -qxf truncated.tar
+"${TARLZ}" -q -xf truncated.tar
 [ $? = 2 ] || test_failed $LINENO
 [ ! -e test.txt ] || test_failed $LINENO
+rm -f truncated.tar || framework_failure
 
+# test compressed and --keep-damaged
+rm -f test.txt || framework_failure
+for i in "${inbad1}" "${inbad2}" ; do
+	"${TARLZ}" -q -xf "${i}.tar.lz"
+	[ $? = 2 ] || test_failed $LINENO "${i}"
+	[ ! -e test.txt ] || test_failed $LINENO "${i}"
+	rm -f test.txt || framework_failure
+	"${TARLZ}" -q -xf "${i}.tar.lz" --keep-damaged
+	[ $? = 2 ] || test_failed $LINENO "${i}"
+	[ -e test.txt ] || test_failed $LINENO "${i}"
+	cmp "${i}" test.txt 2> /dev/null || lzlib_1_11 "$LINENO ${i}"
+	rm -f test.txt || framework_failure
+done
+#
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${bad1_lz}
+"${TARLZ}" -q -xf "${bad1_lz}"
 [ $? = 2 ] || test_failed $LINENO
 [ ! -e foo ] || test_failed $LINENO
-[ -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+cmp cbar bar || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${bad2_lz}
+"${TARLZ}" -q -xf "${bad2_lz}"
 [ $? = 2 ] || test_failed $LINENO
 [ ! -e foo ] || test_failed $LINENO
-[ -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+cmp cbar bar || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${bad3_lz}
+"${TARLZ}" -q -xf "${bad3_lz}"
 [ $? = 2 ] || test_failed $LINENO
-[ -e foo ] || test_failed $LINENO
+cmp cfoo foo || test_failed $LINENO
 [ ! -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${bad4_lz}
+"${TARLZ}" -q -xf "${bad3_lz}" --keep-damaged
+[ $? = 2 ] || test_failed $LINENO
+cmp cfoo foo || test_failed $LINENO
+cmp cbar bar 2> /dev/null || lzlib_1_11 $LINENO
+cmp cbaz baz || test_failed $LINENO
+rm -f foo bar baz || framework_failure
+"${TARLZ}" -q -xf "${bad4_lz}"
 [ $? = 2 ] || test_failed $LINENO
 [ ! -e foo ] || test_failed $LINENO
 [ ! -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${bad5_lz}
+"${TARLZ}" -q -xf "${bad4_lz}" --keep-damaged
+[ $? = 2 ] || test_failed $LINENO
+[ ! -e foo ] || test_failed $LINENO
+cmp cbar bar 2> /dev/null || lzlib_1_11 $LINENO
+cmp cbaz baz || test_failed $LINENO
+rm -f foo bar baz || framework_failure
+"${TARLZ}" -q -xf "${bad5_lz}"
 [ $? = 2 ] || test_failed $LINENO
 [ ! -e foo ] || test_failed $LINENO
 [ ! -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${bad6_lz}
+"${TARLZ}" -q -xf "${bad5_lz}" --keep-damaged
 [ $? = 2 ] || test_failed $LINENO
-[ -e foo ] || test_failed $LINENO
-[ -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+cmp cfoo foo 2> /dev/null || lzlib_1_11 $LINENO
+[ ! -e bar ] || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
+rm -f foo bar baz || framework_failure
+"${TARLZ}" -q -xf "${bad6_lz}"
+[ $? = 2 ] || test_failed $LINENO
+cmp cfoo foo || test_failed $LINENO
+cmp cbar bar || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 
+# test uncompressed and --keep-damaged
+rm -f test.txt || framework_failure
+"${TARLZ}" -q -xf "${inbad1}.tar"
+[ $? = 2 ] || test_failed $LINENO
+[ ! -e test.txt ] || test_failed $LINENO
+rm -f test.txt || framework_failure
+"${TARLZ}" -q -xf "${inbad1}.tar" --keep-damaged
+[ $? = 2 ] || test_failed $LINENO
+[ -e test.txt ] || test_failed $LINENO
+cmp "${inbad1}" test.txt 2> /dev/null || test_failed $LINENO
+rm -f test.txt || framework_failure
+#
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${bad1}
+"${TARLZ}" -q -xf "${bad1}"
 [ $? = 2 ] || test_failed $LINENO
 [ ! -e foo ] || test_failed $LINENO
-[ -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+cmp cbar bar || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${bad2}
+"${TARLZ}" -q -xf "${bad2}"
 [ $? = 2 ] || test_failed $LINENO
-[ -e foo ] || test_failed $LINENO
+cmp cfoo foo || test_failed $LINENO
 [ ! -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${bad3}
+"${TARLZ}" -q -xf "${bad3}"
 [ $? = 2 ] || test_failed $LINENO
-[ -e foo ] || test_failed $LINENO
-[ -e bar ] || test_failed $LINENO
+cmp cfoo foo || test_failed $LINENO
+cmp cbar bar || test_failed $LINENO
 [ ! -e baz ] || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${bad4}
+"${TARLZ}" -q -xf "${bad4}"
 [ $? = 2 ] || test_failed $LINENO
-[ -e foo ] || test_failed $LINENO
+cmp cfoo foo || test_failed $LINENO
 [ ! -e bar ] || test_failed $LINENO
 [ ! -e baz ] || test_failed $LINENO
 rm -f foo bar baz || framework_failure
-"${TARLZ}" -qxf ${bad5}
+"${TARLZ}" -q -xf "${bad5}"
 [ $? = 2 ] || test_failed $LINENO
-[ -e foo ] || test_failed $LINENO
-[ -e bar ] || test_failed $LINENO
-[ -e baz ] || test_failed $LINENO
-rm -f foo bar baz || framework_failure
+cmp cfoo foo || test_failed $LINENO
+cmp cbar bar || test_failed $LINENO
+cmp cbaz baz || test_failed $LINENO
+rm -f cfoo cbar cbaz foo bar baz || framework_failure
+#
+rm -f test3.tar.lz || framework_failure
+"${TARLZ}" -q -xf "${tlzit1}"
+[ $? = 2 ] || test_failed $LINENO
+[ ! -e foo ] || test_failed $LINENO
+[ ! -e bar ] || test_failed $LINENO
+[ ! -e baz ] || test_failed $LINENO
+[ ! -e test3.tar.lz ] || test_failed $LINENO
+rm -f foo bar baz test3.tar.lz || framework_failure
+"${TARLZ}" -q -xf "${tlzit2}"
+[ $? = 2 ] || test_failed $LINENO
+[ ! -e foo ] || test_failed $LINENO
+[ ! -e bar ] || test_failed $LINENO
+[ ! -e baz ] || test_failed $LINENO
+cmp "${test3_lz}" test3.tar.lz || test_failed $LINENO
+rm -f foo bar baz test3.tar.lz || framework_failure
 
 echo
 if [ ${fail} = 0 ] ; then
